@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 /***************************************************************************
                               -------------------
@@ -24,7 +23,7 @@ from qgis.PyQt.QtNetwork import QNetworkProxy
 from .ili2dbutils import get_all_modeldir_in_path
 
 
-class BaseConfiguration(object):
+class BaseConfiguration:
     def __init__(self):
         self.super_pg_user = "postgres"
         self.super_pg_password = "postgres"
@@ -60,7 +59,7 @@ class BaseConfiguration(object):
         self.debugging_enabled = settings.value("DebuggingEnabled", False, bool)
         self.logfile_path = settings.value("LogfilePath", "", str)
 
-    def to_ili2db_args(self, with_modeldir=True):
+    def to_ili2db_args(self, with_modeldir=True, with_usabilityhub_repo=False):
         """
         Create an ili2db command line argument string from this configuration
         """
@@ -74,6 +73,14 @@ class BaseConfiguration(object):
                 ]
                 str_model_directories = ";".join(str_model_directories)
                 args += ["--modeldir", str_model_directories]
+        if with_usabilityhub_repo:
+            if not self.custom_model_directories_enabled:
+                # Workaround for https://github.com/opengisch/QgisModelBaker/issues/784.
+                # Can be removed when ili2db has access to the UsabILIty Hub repository.
+                args += [
+                    "--modeldir",
+                    "%ILI_FROM_DB;%XTF_DIR;http://models.interlis.ch/;%JAR_DIR;https://models.opengis.ch/",
+                ]
         if self.debugging_enabled and self.logfile_path:
             args += ["--trace"]
             args += ["--log", self.logfile_path]
@@ -103,7 +110,7 @@ class BaseConfiguration(object):
         return dirs
 
 
-class Ili2DbCommandConfiguration(object):
+class Ili2DbCommandConfiguration:
     def __init__(self):
         self.base_configuration = BaseConfiguration()
 
@@ -147,7 +154,9 @@ class Ili2DbCommandConfiguration(object):
         # Valid ili file, don't pass --modeldir (it can cause ili2db errors)
         with_modeldir = not self.ilifile
 
-        args = self.base_configuration.to_ili2db_args(with_modeldir=with_modeldir)
+        args = self.base_configuration.to_ili2db_args(
+            with_modeldir=with_modeldir, with_usabilityhub_repo=bool(self.metaconfig_id)
+        )
 
         proxy = QgsNetworkAccessManager.instance().fallbackProxy()
         if proxy.type() == QNetworkProxy.HttpProxy:
@@ -247,6 +256,7 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
 
         if self.disable_validation:
             self.append_args(args, ["--sqlEnableNull"], force_append=True)
+            self.append_args(args, ["--sqlColsAsText"], force_append=True)
         else:
             self.append_args(args, ["--createNumChecks"], True)
             self.append_args(args, ["--createUnique"], True)
@@ -346,12 +356,19 @@ class ImportDataConfiguration(SchemaImportConfiguration):
         if self.baskets:
             self.append_args(args, ["--baskets", ";".join(self.baskets)])
 
-        self.append_args(
-            args,
-            SchemaImportConfiguration.to_ili2db_args(
-                self, extra_args=extra_args, with_action=False
-            ),
-        )
+        if self.with_schemaimport:
+            self.append_args(
+                args,
+                SchemaImportConfiguration.to_ili2db_args(
+                    self, extra_args=extra_args, with_action=False
+                ),
+            )
+        else:
+            self.append_args(args, extra_args)
+
+            self.append_args(
+                args, Ili2DbCommandConfiguration.to_ili2db_args(self), force_append=True
+            )
 
         self.append_args(args, [self.xtffile])
 
